/* Batch Master -- module 4: video & timer routines */

#include "batm.h"

/* Note: if you use Quick C 2.0, turn off pointer checks when you
 * compile this module.
 */

void get_vid_parms()						/* Assumes text mode				*/
{																					
	char far *fp;
	ireg.h.ah = 0x0f;
	int86(0x10, &ireg, &oreg);			/* Get video mode and info 	*/
	vid_mode = (unsigned) oreg.h.al;
	vid_page = (unsigned) oreg.h.bh;
	vid_cols = (unsigned) oreg.h.ah;
	if (vid_mode == 7)
		vid_seg = 0xb000;
	else 
		vid_seg = 0xb800;					/* Get rows & attribute data 	*/
	vid_rows = (unsigned) *((char far *) 0x00400084L) + 1;
	vid_start = *((unsigned far *) 0x0040004eL);
	FP_SEG(fp) = vid_seg;
	FP_OFF(fp) = vid_start + 1;
	vid_attr = (unsigned) *fp;
	ireg.h.ah = 0x03;
	ireg.h.bh = (char) vid_page;
	int86(0x10, &ireg, &oreg);			/* Get cursor information		*/
	curs_row = (int) oreg.h.dh;
	curs_col = (int) oreg.h.dl;
}

void set_cursor(int col, int row)	/* Uses 0-based counts */
{
if (col >= vid_cols || col < 0 || row >= vid_rows || row < 0)
	return;
#ifdef ANSI
	fprintf(stdout,"\x1b[%d;%dH",row+1,col+1);
#else
	ireg.h.ah = 0x02;
	ireg.h.bh = (char) vid_page;
	ireg.h.dh = (char) row;
	ireg.h.dl = (char) col;
	int86(0x10, &ireg, &oreg);
#endif
curs_row = row;
curs_col = col;
}

void set_color(int fore, int back)	/* Set new video attributes */
{
#ifdef ANSI
	fore &= 7;
	back &= 7;
	fprintf(stdout,"\x1b[2;%d;%dm",fore + 30, back + 40);
#else
	fore &= 0xf;
	back &= 0xf;
#endif
	vid_attr = (back << 4) | fore;
}

void show_char(int ch)					/* One char. on screen		 */
{
#ifdef ANSI
	if (ch == '\n') {
		putch('\r');  						/* Expand LF to CR/LF		*/
		putch('\n');
		}
	else
		putch(ch);
#else											/* If direct (non-ANSII)	*/
	if (ch == '\n') {						/* If new line					*/
		if (++curs_row >= vid_rows) {	/* If end of screen			*/
			scroll_up();
			curs_row = vid_rows-1;
			}
		set_cursor(0, curs_row);		/* Set new position			*/
		}
	else if (ch == '\t')					/* If TAB						*/
		do { vid_out(' ');				/* Send enough spaces		*/
			} while (curs_col % 8);
	else vid_out(ch);
#endif
}

void vid_out(int ch)						/* Send 1 char directly		*/
{
#ifndef ANSI
	ireg.h.ah = 9;
	ireg.h.al = (char) ch;
	ireg.h.bh = (char) vid_page;
	ireg.h.bl = (char) vid_attr;
	ireg.x.cx = 1;
	int86(0x10, &ireg, &oreg);			/* Use BIOS to print			*/
	if (++curs_col > vid_cols) {		/* If past end of line		*/
		curs_col = 0;						
		curs_row++;
		}
	if (curs_row >= vid_rows) {
		scroll_up();
		curs_row--;
		}
	set_cursor(curs_col,curs_row);
#endif
}

void show_line(char *s, int lf)		/* Print a line (+ a LF)	*/
{
	while (*s)
		show_char(*s++);
	if (lf)
		show_char('\n');
}

void scroll_up()
{
	ireg.x.ax = 0x0601;
	ireg.h.bh = (char) vid_attr;
	ireg.x.cx = 0;
	ireg.h.dh = (char) vid_rows - 1;
	ireg.h.dl = (char) vid_cols - 1;
	int86(0x10, &ireg, &oreg);
}

void ms_delay(clock_t count)
{
	clock_t target;
	target = count + clock();
	while (clock() < target)
		;
}

void wait_time(char *s)
{
	char hr, min;

	if (strlen(s) < 5 || *(s+2) != ':')
		return;
	hr = (char) atoi(s);
	min= (char) atoi(s+3);
	if (hr > 23 || hr < 0 || min > 59 || min < 0)
		return;
	do {
		ireg.h.ah = 0x2c;
		intdos(&ireg, &oreg);
		} while( oreg.h.ch != hr && oreg.h.cl != min );
}

void make_beep(unsigned freq, unsigned dur)
{
	int control;
	if (freq)
		{
		dur = (unsigned) ((float)dur * CLK_TCK / 10.0);
		outp( 0x43, 0xb6);
		freq = (unsigned) (1193180L / freq);
		outp( 0x42, (char) freq);
		outp( 0x42, (char) (freq >> 8));
		control = inp(0x61);
		outp( 0x61, control | 0x3);
		}
	ms_delay ((clock_t) dur);
	if (freq)
		outp( 0x61, control);
}


