comment |
   Assembly language version of FREE
   Written for MASM 5.1 and TASM 1.0
   Remove the comment marks for the first two lines
   if you are using TASM.
   Save as FREE.ASM
   For MASM:    MASM FREE;
   		LINK FREE;
		EXE2BIN FREE.EXE FREE.COM

   For TASM:    TASM FREE;
   		TLINK FREE;
		EXE2BIN FREE.EXE FREE.COM

   Ignore the "no stack segment" warning that LINK or TLINK will
   generate.
   |

   ;		MASM51		
   ;		QUIRKS

CR		equ	0dh
LF		equ	0ah

code	segment
	assume cs:code, ds:code, es:code
	org	100h
start:	jmp	begin			;Jump around data

print	macro	msg
	mov	dx,offset msg
	mov	ah,09h
	int	21h
	endm

total_mem	dd	?
free_mem	dd	?

dec_table	dd	100000000
		dd	 10000000
		dd	  1000000
		dd	   100000
		dd	    10000
		dd	     1000
		dd	      100
		dd	       10
		dd	        1

hex_table	db	'0123456789ABCDEF'
psp		dw	?
drive		db	?

msg1		db	LF,CR,"Drive "
msg1_d		db	" :$"

msg2		db	LF,CR,"  "
msg2_d		db	"         "
msg2_t		db	" bytes total disk space$"

msg3		db	LF,CR,"  "
msg3_d		db	"         "
msg3_t		db	" bytes available on disk",LF,CR,"$"

msg4		db	LF,CR,"  "
msg4_d		db	"         "
msg4_t		db	" bytes total memory$"

msg5		db	LF,CR,"  "
msg5_d		db	"         "
msg5_t		db	" bytes free",LF,CR,"$"

msg6		db	LF,CR,"  Next program will load at "
msg6_d		db	"0000:0000 hex",LF,CR,"$"

msg7		db	LF,CR,"Invalid drive",LF,CR,"$"

begin:	mov	ax,cs			;code in AX
	mov	ds,ax			;Set up DS
	mov	[psp],es		;Save PSP segment
	test	byte ptr es:[80h],-1	;Any command tail?
	jnz	@F			;Yes -- use it
	call	get_default_drive	;Else ask DOS for drive
	jmp	short info		;  and get to work

  @@:	mov	[drive],-1		;Prepare to fail
  	mov	si,81h			;ES:SI ==> command tail
  @@:	cmp	byte ptr es:[si],' '	;Leading space?
  	ja	@F			;No -- we have something to look at
	inc	si			;Else bump pointer
	jmp	@B			;  and look again
  @@:	cmp	byte ptr es:[si+1],':'	;Valid drive designator
  	jne	info			;No -- we've failed
	mov	al,es:[si]		;Else get drive letter
	and	al,0dfh			;Convert to uppercase if needed
	sub	al,'A'-1		;Convert to number
	mov	[drive],al		;And save it

info:	push	ds			;Copy our segment
	pop	es			;  into ES
	call	get_drive_info		;Display drive stats
	call	get_memory_info		;And memory stats
	mov	ax,4c00h		;Return to DOS
	int	21h			;That's all, folks!

get_default_drive	proc
	mov	ah,19h			;DOS function: get default drive
	int	21h			;Call DOS
	inc	al			;Put in 1-x range
	mov	[drive],al		;Save it
	ret
get_default_drive	endp


get_drive_info	proc
	mov	ah,36h			;DOS function: get drive data
	mov	dl,[drive]		;Drive letter in DL
	int	21h			;Call DOS
	cmp	ax,-1			;Invalid drive?
	jne	@F			;It's okay -- get to work
	print	msg7			;Else print error
	ret				;And return
  @@:	push	bx			;Save free clusters
  	push	dx			;  and total clusters
	push	ax			;Save sectors per cluster
	push	cx			;  and bytes per sector
	mov	al,[drive]		;Get drive number
	add	al,'A'-1		;Change to letter
	mov	[msg1_d],al		;Save in msg
	print	msg1			;And print the message

	pop	cx			;Get sector size
	pop	ax			;And cluster size
	mul	cx			;DX/AX = bytes per cluster
	pop	cx			;CX = total cluster count
	push	dx			;Save bytes per cluster
	push	ax			;  for later
	call	mul32_16	   	;Get total bytes in DX/AX
	mov	si,offset msg2_d	;SI ==> data area
	call	long_2_ASCII		;Change DX/AX to ASCII
	print	msg2			;Print total size

	pop	ax			;DX/AX = bytes per cluster
	pop	dx
	pop	cx			;CX = free clusters
	call	mul32_16	   	;Get bytes free in DX/AX
	mov	si,offset msg3_d	;SI ==> data area
	call	long_2_ASCII		;Change DX/AX to ASCII
	print	msg3			;Print free space
	ret
get_drive_info	endp

;  Multiply 32 bits in DX/AX by 16 bits in CX
;  Return 32-bit value in DX/AX.  Ignore overflow.
;  Uses BX
mul32_16	proc
	push	ax			;Save low word of multiplicand
	mov	ax,dx			;High word in AX
	mul	cx			;DX/AX = CX * high word
	mov	bx,ax			;Save least-signif. 16 bits
	pop	ax			;AX = low word of multiplicand
	mul	cx			;DX/AX = CX * low word
	add	dx,bx			;Add in 1st partial product
	ret
mul32_16	endp

;  Convert value in DX/AX to decimal ASCII and store at DS:[SI]
;  Values in DX/AX and SI are destroyed.  Maximum output size: 9 bytes.
;  Uses CX, DI
long_2_ASCII	proc
	push	si			;Save output pointer
	mov	di,offset dec_table
	mov	cx,9			;Maximum bytes
a_digit:
	mov	byte ptr [si],'0'	;We have to start somewhere
  @@:	sub	ax,[di]			;Subtract power of 10 from
  	sbb	dx,[di+2]		;  DX/AX
	jc	@F			;Go if underflow
	inc	byte ptr [si]		;Else bump digit
	jmp	@B			;  and do it again
  @@:	add	ax,[di]			;Correct underflow
  	adc	dx,[di+2]
	inc	si			;SI ==> next digit position
	add	di,4			;DI ==> next power of 10
	loop	a_digit			;Do for all 9 digits

	pop	si			;SI ==> beginning of output
	mov	cx,8			;Up to 8 leading 0's
  @@:	cmp	byte ptr [si],'0'	;Leading 0?
  	ja	@F			;No -- we're done
	mov	byte ptr [si],' '	;Else change to space
	inc	si			;Point to next
	loop	@B			;And try again
  @@:	ret				;All done
long_2_ASCII	endp

get_memory_info	proc
	int	12h			;Get memory size
	mov	cx,64			;Paragraphs per KB
	mul	cx			;AX = paragraphs of memory
	push	ax			;Save for later
	mov	cx,16			;Bytes per paragraph
	mul	cx			;DX/AX = total memory in bytes
	mov	si,offset msg4_d	;SI ==> space for result
	call	long_2_ASCII		;Convert to ASCII
	print	msg4			;Print result
	pop	ax			;AX = total paragraphs
	sub	ax,[psp]		;Subtract start of this program
	inc	ax			;Add 1 paragraph for Arena Header
	mov	cx,16			;Bytes per paragraph
	mul	cx			;DX/AX = bytes free
	mov	si,offset msg5_d	;SI ==> space for result
	call	long_2_ASCII		;Convert to ASCII
	print	msg5			;Print it

	mov	ax,[psp]		;Get PSP again
	mov	di,offset msg6_d	;SI ==> space for result
	call	int_to_hex		;Convert AX to ASCII Hex
	print	msg6			;Print result
	ret
get_memory_info	endp

; Convert value in AX to ASCII Hex
; Store result in 4 bytes at ES:[DI]
; Uses BX, CX, destroys AX & DI

int_to_hex	proc
	mov	bx,offset hex_table	;BX ==> translation table
	mov	cl,4			;For shift operations
	push	ax			;Save value in AX
	xchg	al,ah			;High byte in AL
	push	ax			;Save it for a moment
	shr	al,cl			;High nibble to low 4 bits
	xlat				;Hex digit in AL
	stosb				;Put into result
	pop	ax			;Get back reversed value
	and	al,0fh			;2nd nibble in AL
	xlat				;Get hex digit
	stosb				;  and store it
	pop	ax			;AX = original value
	push	ax			;Save once more
	shr	al,cl			;3rd nibble in AL
	xlat				;Change to hex digit
	stosb				;Put in output
	pop	ax			;Get value once again
	and	al,0fh			;Low nibble in AL
	xlat				;Change to hex
	stosb				;Store digit #4
	ret
int_to_hex	endp
code		ends
	end	start
